@@ -128,31 +128,21 @@ def frac_gradient(t, frac):
 
 # vqgan vae
 
-class VQGanVAETrainer(nn.Module):
+class LayerNormChan(nn.Module):
     def __init__(
         self,
-        *,
-        vae,
-        lr = 3e-4
+        dim,
+        eps = 1e-5
     ):
         super().__init__()
-        assert isinstance(vae, VQGanVAE), 'vae must be instance of VQGanVAE'
+        self.eps = eps
+        self.g = nn.Parameter(torch.ones(1, dim, 1, 1))
+        self.b = nn.Parameter(torch.zeros(1, dim, 1, 1))
 
-        self.vae = vae
-        self.optim = Adam(vae.parameters(), lr = lr)
-        self.register_buffer('state', torch.ones((1,), dtype = torch.bool))
-
-    def forward(self, img):
-        return_loss_key = 'return_loss' if self.state else 'return_discr_loss'
-        vae_kwargs = {return_loss_key: True}
-
-        loss = self.vae(img, **vae_kwargs)
-        loss.backward()
-        self.optim.step()
-        self.optim.zero_grad()
-
-        self.state = self.state.data.copy_(~self.state)
-        return loss, bool(self.state)
+    def forward(self, x):
+        std = torch.var(x, dim = 1, unbiased = False, keepdim = True).sqrt()
+        mean = torch.mean(x, dim = 1, keepdim = True)
+        return (x - mean) / (std + self.eps) * self.g + self.b
 
 class Discriminator(nn.Module):
     def __init__(
@@ -181,6 +171,31 @@ class Discriminator(nn.Module):
 
         return self.to_logits(x)
 
+class ConvNextBlock(nn.Module):
+    def __init__(
+        self,
+        dim,
+        act = nn.ReLU(),
+        ds_kernel_size = 3,
+        mult = 4,
+    ):
+        """
+        https://arxiv.org/abs/2201.03545
+        """
+        super().__init__()
+        inner_dim = dim * mult
+
+        self.net = nn.Sequential(
+            nn.Conv2d(dim, dim, ds_kernel_size, padding = ds_kernel_size // 2, groups = dim),
+            LayerNormChan(dim),
+            nn.Conv2d(dim, inner_dim, 1),
+            act,
+            nn.Conv2d(inner_dim, dim, 1)
+        )
+
+    def forward(self, x):
+        return self.net(x) + x
+
 class ResBlock(nn.Module):
     def __init__(self, chan):
         super().__init__()
